{*******************************************************}
{                                                       }
{       DataSet combobox component                      }
{                                                       }
{       by Oleg Gopaniouk                               }
{                                                       }
{       Email : gopaniou@hq.kvazar-micro.com            }
{                                                       }
{       Creation date is 29.03.1998                     }
{*******************************************************}
unit DSComboBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,dbtables,db,dbctrls,DsgnIntf;

type
  TDSComboBox = class(TCustomComboBox)
  private
    { Private declarations }
    FQuery:TQuery;
    FDataBaseName:String;
    FDisplayedField:String;
    function GetActive:Boolean;
    procedure SetActive(Status:Boolean);
    property Items;
    procedure SetSql(Sql:TStringList);
    function  GetSql:TStringList;
  protected
    { Protected declarations }
  public
    { Public declarations }
    procedure Open; // Open query and fill Combobox
    procedure Close; //Close query and clear Combobox
    procedure Refresh;
    procedure Loaded;override;
    function Find(const KeyFields: string; const KeyValues: Variant; Options: TLocateOptions): Boolean;
    procedure FindSet(const KeyFields: string; const KeyValues: Variant; Options: TLocateOptions);
    function FieldByName(FieldName:String):TField;//Get value by field name
    function ParamByName(ParamName:String):TParam;// Get or get parameters value
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;override;
    property Active: Boolean read GetActive write SetActive;
  published
    { Published declarations }
    property Style; {Must be published before Items}
    property DataBaseName: String read FDataBaseName write FDataBaseName;//Set or get database name
    property SQL:TStringList read GetSQL write SetSQL;
    property DiplayedField:String read FDisplayedField write FDisplayedField;// Set or get field which displayed in combobox
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
    property OnStartDrag;
    property Text;
    property Visible;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Font;
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property DropDownCount;
    property Enabled;
  end;

  TDBNamePropEditor = class(TStringProperty)//Property editor for DataBaseName property
  public
        procedure GetValues(Proc:TGetStrProc);override;
        function  GetAttributes:TPropertyAttributes;override;
        function  GetName:string;override;
  end;
procedure Register;

implementation

procedure TDSComboBox.Loaded;
begin
inherited Loaded;
FQuery.DatabaseName:=FDataBaseName;
end;


constructor TDSComboBox.Create(AOwner:TComponent);
begin
inherited Create(AOwner);
FQuery:=TQuery.Create(self);
end;

destructor TDSComboBox.Destroy;
begin
FQuery.Close;
FQuery.Free;
inherited Destroy;
end;

function TDSComboBox.ParamByName(ParamName:String):TParam;//Set parameters' value
begin
Result:=FQuery.ParamByName(ParamName);
end;

procedure TDSComboBox.Open;//Open DSCombobox
begin
   if not FQuery.Prepared then FQuery.Prepare;
   FQuery.Open;
   FQuery.First;
   Items.Clear;
   While not FQuery.eof do
     begin
         Items.Add(FQuery.FieldByName(FDisplayedField).AsString);
         FQuery.Next;
     end;
end;

procedure TDSComboBox.SetSql(SQL:TStringList);//Set SQL property
begin
FQuery.SQL.Assign(SQL);
end;

function  TDSComboBox.GetSql:TStringList;//Return SQL property
begin
Result:=TStringList(FQuery.SQL);
end;

procedure TDSComboBox.Close;//Close DSCombobox
begin
FQuery.close;
Items.Clear;
end;

function TDSComboBox.FieldByName(FieldName:String):TField;//Get field value by it's name
var Temp:TField;
begin
FQuery.First;
FQuery.MoveBy(ItemIndex);
Result:=FQuery.FieldByName(FieldName);
end;


function TDSComboBox.Find(const KeyFields: string; const KeyValues: Variant; Options: TLocateOptions): Boolean;
begin
if Active then
Result:=FQuery.Locate(KeyFields,KeyValues,Options)
else
Raise Exception.Create('No array !')
end;

procedure TDSComboBox.SetActive(Status:Boolean);
begin
if Status then begin
   Open;
end
else Close;
end;

procedure TDSComboBox.Refresh;
var TempParams:TParams;
begin
TempParams:=TParams.Create;
try
   TempParams.Assign(FQuery.Params);
   Close;
   FQuery.Open;
   FQuery.First;
   While not FQuery.eof do
     begin
         Items.Add(FQuery.FieldByName(FDisplayedField).AsString);
         FQuery.Next;
     end;
Except
      TempParams.Free;
end;
TempParams.Free;
end;

procedure TDSComboBox.FindSet(const KeyFields: string; const KeyValues: Variant; Options: TLocateOptions);
begin
if Active then begin
FQuery.Locate(KeyFields,KeyValues,Options);
ItemIndex:=Items.IndexOf(FQuery.FieldByname(FDisplayedField).AsString)
end
else Raise Exception.Create('No array !');
end;

function TDSComboBox.GetActive:Boolean;
begin
Result:=FQuery.Active;
end;

procedure TDBNamePropEditor.GetValues(Proc:TGetStrProc);//Get list all uknown databases
var
  List:TStringList;
  i:Integer;
begin
      Session.Active:=True;
      List:=TStringList.Create;
      Session.GetDatabaseNames(List);
      for i :=0  to List.Count-1 do  Proc(List.Strings[i]);
      List.Free;
      Session.Active:=False;
end;

function  TDBNamePropEditor.GetAttributes:TPropertyAttributes;
begin
Result:=[paValueList,paSortList];
end;

function  TDBNamePropEditor.GetName:string;
begin
Result:='DataBaseName';
end;

procedure Register;//Register DSCombobox and Property Editor
begin
  RegisterPropertyEditor(TypeInfo(String),TDSComboBox,'DataBaseName',TDBNamePropEditor);
  RegisterComponents('Data Controls', [TDSComboBox]);
end;

end.
